@@ -251,7 +251,8 @@ void SessionWidget::slotCheck()
     
     if(!url.protocol().startsWith("http")) 
     {
-        DocumentRootDialog dialog(this, url.directory());
+        QString documentRootHint = url.directory().isEmpty() ? "/" : url.directory();
+        DocumentRootDialog dialog(this, documentRootHint);
         dialog.exec();
         search_manager_->setDocumentRoot(KURL::fromPathOrURL(dialog.url()));
     }
@@ -494,9 +495,12 @@ void SessionWidget::insertUrlAtCombobox(QString const& url)
     combobox_url->addToHistory(url);
 }
 
-void SessionWidget::showBottomStatusLabel(QListViewItem * item)
+void SessionWidget::showBottomStatusLabel(QListViewItem* item)
 {
     kdDebug(23100) << "SessionWidget::showBottomStatusLabel" << endl;
+    
+    if(!item)
+        return;
 
     TreeViewItem* _item = tree_view->myItem(item);
     if(_item)
