@@ -27,13 +27,15 @@
 #include <ktexteditor/highlightinginterface.h>
 #include <ktexteditor/popupmenuinterface.h>
 #include <ktexteditor/editorchooser.h>
+#include <ktexteditor/markinterface.h>
+#include <kio/netaccess.h>
 
 
 //app includes
-#ifdef BUILD_KAFKAPART
 #include "undoredo.h"
 #include "wkafkapart.h"
-#endif
+#include "tagactionmanager.h"
+#include "tagactionset.h"
 
 #include "whtmlpart.h"
 #include "document.h"
@@ -42,10 +44,17 @@
 #include "project.h"
 #include "resource.h"
 #include "quanta.h"
+#include "quantabookmarks.h"
 #include "toolbartabwidget.h"
 #include "parser.h"
 #include "qextfileinfo.h"
 #include "qpevents.h"
+#include "ksavealldialog.h"
+#include "annotationoutput.h"
+
+#ifdef ENABLE_CVSSERVICE
+#include "cvsservice.h"
+#endif
 
 #define SEPARATOR_INDEX 3
 #define RELOAD_ID 11
@@ -62,20 +71,26 @@ ViewManager::ViewManager(QObject *parent, const char *name) : QObject(parent, na
     m_tabPopup->insertItem(i18n("Close &Other Tabs"), this, SLOT(slotCloseOtherTabs()));
     m_tabPopup->insertItem(i18n("Close &All"), this, SLOT(closeAll()));
     m_tabPopup->insertItem(SmallIcon("revert"), i18n("&Reload"), this, SLOT(slotReloadFile()), 0, RELOAD_ID);
-    m_tabPopup->insertItem(SmallIcon("up"), i18n("&Upload File..."), this, SLOT(slotUploadFile()), 0, UPLOAD_ID);
+    m_tabPopup->insertItem(SmallIcon("up"), i18n("&Upload File"), this, SLOT(slotUploadFile()), 0, UPLOAD_ID);
     m_tabPopup->insertItem(SmallIcon("editdelete"), i18n("&Delete File"), this, SLOT(slotDeleteFile()), 0, DELETE_ID);
     m_tabPopup->insertSeparator();
     m_fileListPopup = new KPopupMenu(quantaApp);
     connect(m_fileListPopup, SIGNAL(aboutToShow()), this, SLOT(slotFileListPopupAboutToShow()));
     connect(m_fileListPopup, SIGNAL(activated(int)), this, SLOT(slotFileListPopupItemActivated(int)));
+    m_bookmarks = new QuantaBookmarks(this, QuantaBookmarks::Position, true);
+    m_bookmarksMenu = new KPopupMenu(quantaApp);
+    m_bookmarks->setBookmarksMenu(m_bookmarksMenu);
+    connect(m_bookmarks, SIGNAL(gotoFileAndLine(const QString&, int, int)), quantaApp, SLOT(gotoFileAndLine(const QString&, int, int)));
+    m_bookmarksMenuId = m_tabPopup->insertItem(SmallIconSet("bookmark"), i18n("&Bookmarks"), m_bookmarksMenu);
     m_tabPopup->insertItem(i18n("&Switch To"), m_fileListPopup);
     m_contextView = 0L;
+    m_cvsMenuId = -1;
     m_separatorVisible = false;
 }
 
-QuantaView* ViewManager::createView()
+QuantaView* ViewManager::createView(const QString &caption)
 {
-    QuantaView *view = new QuantaView(quantaApp);
+    QuantaView *view = new QuantaView(quantaApp, "", caption);
     quantaApp->addWindow(view);
     connect(view, SIGNAL(cursorPositionChanged()), quantaApp, SLOT(slotNewLineColumn()));
     connect(view, SIGNAL(title(const QString &)), quantaApp, SLOT(slotNewLineColumn()));
@@ -91,10 +106,10 @@ QuantaView* ViewManager::createView()
 
 void ViewManager::createNewDocument()
 {
-   QuantaView *view = createView();
-   int i = 1;
-   while ( isOpened(KURL("file:/"+i18n("Untitled%1").arg(i)))) i++;
+  int i = 1;
+  while (isOpened(KURL("file:"+i18n("Untitled%1").arg(i)))) i++;
   QString fname = i18n("Untitled%1").arg(i);
+  QuantaView *view = createView(fname);
 
 #ifdef ENABLE_EDITORS
   KTextEditor::Document *doc =
@@ -106,6 +121,7 @@ void ViewManager::createNewDocument()
   KTextEditor::Document *doc = KTextEditor::createDocument ("libkatepart", view, "KTextEditor::Document");
 #endif
   Document *w = new Document(doc, 0L);
+  connect(w, SIGNAL(showAnnotation(uint, const QString&, const QPair<QString, QString>&)), quantaApp->annotationOutput(), SLOT(insertAnnotation(uint, const QString&, const QPair<QString, QString>&)));
   QString encoding = quantaApp->defaultEncoding();
   KTextEditor::EncodingInterface* encodingIf = dynamic_cast<KTextEditor::EncodingInterface*>(doc);
   if (encodingIf)
@@ -134,9 +150,24 @@ void ViewManager::createNewDocument()
   quantaApp->newCursorPosition("", 1 , 1);
 }
 
-bool ViewManager::removeView(QuantaView *view, bool force)
+bool ViewManager::removeView(QuantaView *view, bool force, bool createNew)
 {
     if (!view) return false;
+    int noOfViews = 0;
+    QValueList<Document*> list;
+    KMdiIterator<KMdiChildView*> *it = quantaApp->createIterator();
+    for (it->first(); !it->isDone(); it->next())
+    {
+      noOfViews++;
+      if (noOfViews > 1)
+        break;
+    }
+    delete it;
+    if (noOfViews == 1 && view->document()->isUntitled() && !view->document()->isModified() && createNew)
+    {
+      quantaApp->slotShowPreviewWidget(false);
+      return true;
+    }
     bool mayRemove = view->mayRemove();
     if (mayRemove)
     {
@@ -150,12 +181,17 @@ bool ViewManager::removeView(QuantaView *view, bool force)
             m_lastActiveEditorView = 0L;
         if (view == activeView())
             ToolbarTabWidget::ref()->reparent(0L, 0, QPoint(), false);
+        if (!createNew)
+          disconnect(quantaApp, SIGNAL(lastChildViewClosed()), this, SLOT(slotLastViewClosed()));
         quantaApp->closeWindow(view);
-        if (allEditorsClosed())
+        if (createNew)
         {
-            createNewDocument();
-            quantaApp->slotNewStatus();
-        }
+          if (allEditorsClosed())
+          {
+            quantaApp->slotFileNew();
+          }
+        } else
+          connect(quantaApp, SIGNAL(lastChildViewClosed()), this, SLOT(slotLastViewClosed()));
         return true;
        }
     }
@@ -170,7 +206,7 @@ Document *ViewManager::activeDocument()
 
 QuantaView* ViewManager::activeView()
 {
-    return dynamic_cast<QuantaView *>(quantaApp->activeWindow());
+    return static_cast<QuantaView *>(quantaApp->activeWindow());
 }
 
 void ViewManager::slotViewActivated(KMdiChildView *view)
@@ -180,12 +216,13 @@ void ViewManager::slotViewActivated(KMdiChildView *view)
    if (m_lastActiveView)
    {
      m_lastActiveView->deactivated();
-     Document *w = static_cast<QuantaView*>(m_lastActiveView)->document();
-      quantaApp->restoreFromTempfile(w);
    }
    QuantaView *qView = static_cast<QuantaView*>(view);
    qView->activated();
 
+#ifdef DEBUG_PARSER
+//   kdDebug(24000) << "Calling clearGroups from ViewManager::slotViewActivated" << endl;
+#endif
   parser->clearGroups();
   parser->setSAParserEnabled(true);
   quantaApp->reparse(true); //FIXME
@@ -197,17 +234,23 @@ void ViewManager::slotViewActivated(KMdiChildView *view)
   {
     m_lastActiveEditorView = m_lastActiveView;
   }
+
   Document *w = qView->document();
   if (w && !w->isUntitled())
   {
     emit viewActivated(w->url());
+
+    bool flag = TagActionManager::canIndentDTD(w->defaultDTD()->name);
+    quantaApp->actionCollection()->action("apply_source_indentation")->setEnabled(flag);
   }
 }
 
 void ViewManager::slotCloseOtherTabs()
 {
   KMdiChildView *currentView;
-   if (m_contextView)
+  KMdiChildView *view;
+  QuantaView *qView;
+  if (m_contextView)
      currentView = m_contextView;
    else
      currentView = quantaApp->activeWindow();
@@ -219,8 +262,53 @@ void ViewManager::slotCloseOtherTabs()
       children.append(it->currentItem());
   }
   delete it;
-  KMdiChildView *view;
+  KURL::List modifiedList;
   QValueListIterator<KMdiChildView *> childIt;
+  for (childIt = children.begin(); childIt != children.end(); ++childIt)
+  {
+    view = *childIt;
+    qView = dynamic_cast<QuantaView*>(view);
+    if (qView && view != currentView)
+    {
+      Document *w = qView->document();
+      if (w && w->isModified())
+      {
+        modifiedList += w->url();
+      }
+    }
+  }
+  if (!modifiedList.isEmpty())
+  {
+    KURL::List filesToSave;
+    KSaveSelectDialog dlg(modifiedList, filesToSave /*empty ignore list */, quantaApp);
+    if (dlg.exec() == QDialog::Accepted)
+    {
+      filesToSave = dlg.filesToSave();
+      for (childIt = children.begin(); childIt != children.end(); ++childIt)
+      {
+        view = *childIt;
+        qView = dynamic_cast<QuantaView*>(view);
+        if (view && view != currentView)
+        {
+          Document *w = qView->document();
+          if (w)
+          {
+            if (filesToSave.contains(w->url()))
+              if (!qView->saveModified(false))
+            {
+              return;  //save aborted
+            }
+            w->setModified(false);
+            qView->updateTab();
+          }
+        }
+      }
+    } else
+    {
+      return;  //save aborted
+    }
+  }
+
   for (childIt = children.begin(); childIt != children.end(); ++childIt)
   {
       view = *childIt;
@@ -236,7 +324,7 @@ void ViewManager::slotCloseOtherTabs()
 QuantaView* ViewManager::isOpened(const KURL& url)
 {
   KURL url2 = url;
-  if (url2.isLocalFile())
+  if (url2.isLocalFile() && !url.path().startsWith(i18n("Untitled")))
   {
       QDir dir(url2.path());
       url2.setPath(dir.canonicalPath());
@@ -276,7 +364,26 @@ KURL::List ViewManager::openedFiles(bool noUntitled)
   return list;
 }
 
-bool ViewManager::saveAll(bool dont_ask)
+QValueList<Document*> ViewManager::openedDocuments()
+{
+  QValueList<Document*> list;
+  KMdiIterator<KMdiChildView*> *it = quantaApp->createIterator();
+  QuantaView *view;
+  for (it->first(); !it->isDone(); it->next())
+  {
+    view = dynamic_cast<QuantaView*>(it->currentItem());
+    if (view)
+    {
+      Document *w = view->document();
+      if (w)
+        list.append(w);
+    }
+  }
+  delete it;
+  return list;
+}
+
+bool ViewManager::saveAll()
 {
   bool flagsave = true;
   KMdiIterator<KMdiChildView*> *it = quantaApp->createIterator();
@@ -289,15 +396,11 @@ bool ViewManager::saveAll(bool dont_ask)
           Document *w = view->document();
           if ( w && w->isModified())
           {
-              if (dont_ask && !w->isUntitled())
+              if (!w->isUntitled())
               {
                   emit eventHappened("before_save", w->url().url(), QString::null);
-#ifdef BUILD_KAFKAPART
                   w->docUndoRedo->fileSaved();
-#endif
                   w->save();
-                  w->closeTempFile();
-                  w->createTempFile();
                   w->removeBackup(quantaApp->config());
                   if (w->isModified())
                       flagsave = false;
@@ -316,15 +419,12 @@ bool ViewManager::saveAll(bool dont_ask)
   return flagsave;
 }
 
-void ViewManager::closeAll(bool createNew)
+bool ViewManager::closeAll(bool createNew)
 {
   quantaApp->slotShowPreviewWidget(false);
-  disconnect(quantaApp, SIGNAL(viewActivated (KMdiChildView *)), this, SLOT(slotViewActivated(KMdiChildView*)));
-  disconnect(quantaApp, SIGNAL(lastChildViewClosed()), this, SLOT(slotLastViewClosed()));
-
+  parser->setSAParserEnabled(false);
   KMdiIterator<KMdiChildView*> *it = quantaApp->createIterator();
   QuantaView *view;
-  ToolbarTabWidget::ref()->reparent(0L, 0, QPoint(), false);
   //save the children first to a list, as removing invalidates our iterator
   QValueList<KMdiChildView *> children;
   for (it->first(); !it->isDone(); it->next())
@@ -332,7 +432,56 @@ void ViewManager::closeAll(bool createNew)
       children.append(it->currentItem());
   }
   delete it;
+  KURL::List modifiedList;
   QValueListIterator<KMdiChildView *> childIt;
+  for (childIt = children.begin(); childIt != children.end(); ++childIt)
+  {
+    view = dynamic_cast<QuantaView*>(*childIt);
+    if (view)
+    {
+      Document *w = view->document();
+      if (w && w->isModified())
+      {
+        modifiedList += w->url();
+      }
+    }
+  }
+  if (!modifiedList.isEmpty())
+  {
+    KURL::List filesToSave;
+    KSaveSelectDialog dlg(modifiedList, filesToSave /*empty ignore list */, quantaApp);
+    if (dlg.exec() == QDialog::Accepted)
+    {
+      filesToSave = dlg.filesToSave();
+      for (childIt = children.begin(); childIt != children.end(); ++childIt)
+      {
+        view = dynamic_cast<QuantaView*>(*childIt);
+        if (view)
+        {
+          Document *w = view->document();
+          if (w)
+          {
+            if (filesToSave.contains(w->url()))
+              if (!view->saveModified(false))
+              {
+                emit filesClosed(false);
+                return false;  //save aborted
+              }
+            w->setModified(false);
+            view->updateTab();
+          }
+        }
+      }
+    } else
+    {
+      emit filesClosed(false);
+      return false;  //save aborted
+    }
+  }
+  disconnect(quantaApp, SIGNAL(viewActivated (KMdiChildView *)), this, SLOT(slotViewActivated(KMdiChildView*)));
+  disconnect(quantaApp, SIGNAL(lastChildViewClosed()), this, SLOT(slotLastViewClosed()));
+  ToolbarTabWidget::ref()->reparent(0L, 0, QPoint(), false);
+
   for (childIt = children.begin(); childIt != children.end(); ++childIt)
   {
       view = dynamic_cast<QuantaView*>(*childIt);
@@ -343,11 +492,10 @@ void ViewManager::closeAll(bool createNew)
           {
               if (view->mayRemove())
               {
-                 w->closeTempFile();
                  if (!w->isUntitled() && w->url().isLocalFile())
                  {
                    fileWatcher->removeFile(w->url().path());
-                   kdDebug(24000) << "removeFile[closeAll]: " << w->url().path() << endl;
+//                   kdDebug(24000) << "removeFile[closeAll]: " << w->url().path() << endl;
                  }
                  quantaApp->guiFactory()->removeClient(w->view());
                  if (view == m_lastActiveEditorView)
@@ -357,11 +505,13 @@ void ViewManager::closeAll(bool createNew)
                  quantaApp->closeWindow(view);
               } else
               {
+                //actually this code should be never executed
                connect(quantaApp, SIGNAL(viewActivated (KMdiChildView *)), this, SLOT(slotViewActivated(KMdiChildView*)));
                connect(quantaApp, SIGNAL(lastChildViewClosed()), this, SLOT(slotLastViewClosed()));
                view->activated();
-                return;
-               }
+               emit filesClosed(false);
+                return false;
+              }
           } else
           {
               if (view == m_documentationView)
@@ -377,7 +527,9 @@ void ViewManager::closeAll(bool createNew)
   {
       createNewDocument();
       quantaApp->slotNewStatus();
-   }
+  }
+  emit filesClosed(true);
+  return true;
 }
 
 bool ViewManager::isOneModified()
@@ -414,8 +566,7 @@ QuantaView* ViewManager::documentationView(bool create)
 
 void ViewManager::slotLastViewClosed()
 {
-    createNewDocument();
-    quantaApp->slotNewStatus();
+  quantaApp->slotFileNew();
 }
 
 bool ViewManager::allEditorsClosed()
@@ -439,24 +590,75 @@ void ViewManager::slotTabContextMenu(QWidget *widget, const QPoint& point)
 {
    if (m_separatorVisible)
        m_tabPopup->removeItemAt(SEPARATOR_INDEX);
+   m_separatorVisible = false;
    m_contextView = dynamic_cast<QuantaView*>(widget);
-   if (m_contextView && m_contextView->document())
+   Document *w = 0L;
+   if (m_contextView)
+    w = m_contextView->document();
+   if (w)
    {
-       m_tabPopup->insertSeparator(SEPARATOR_INDEX);
-       m_tabPopup->setItemVisible(RELOAD_ID, true);
-       if (Project::ref()->hasProject() && Project::ref()->contains(m_contextView->document()->url()))
+       if (!w->isUntitled())
+       {
+         m_separatorVisible = true;
+         m_tabPopup->insertSeparator(SEPARATOR_INDEX);
+       }
+       if (Project::ref()->hasProject() && Project::ref()->contains(w->url()))
          m_tabPopup->setItemVisible(UPLOAD_ID, true);
        else
          m_tabPopup->setItemVisible(UPLOAD_ID, false);
-       m_tabPopup->setItemVisible(DELETE_ID, true);
-       m_separatorVisible = true;
+       if (w->isUntitled())
+       {
+         m_tabPopup->setItemVisible(RELOAD_ID, false);
+         m_tabPopup->setItemVisible(DELETE_ID, false);
+       } else
+       {
+         m_tabPopup->setItemVisible(RELOAD_ID, true);
+         m_tabPopup->setItemVisible(DELETE_ID, true);
+       }
    } else
    {
        m_tabPopup->setItemVisible(RELOAD_ID, false);
        m_tabPopup->setItemVisible(UPLOAD_ID, false);
        m_tabPopup->setItemVisible(DELETE_ID, false);
-       m_separatorVisible = false;
    }
+   bool bookmarksFound = false;
+   if (w && w->markIf)
+   {
+     m_bookmarks->setDocument(w);
+     QPtrList<KTextEditor::Mark> m = w->markIf->marks();
+     QPtrListIterator<KTextEditor::Mark> it(m);
+     for(; *it; ++it)
+     {
+       if ((*it)->type & KTextEditor::MarkInterface::markType01)
+       {
+         bookmarksFound = true;
+         break;
+       }
+     }
+   }
+   m_tabPopup->setItemVisible(m_bookmarksMenuId, bookmarksFound);
+#ifdef ENABLE_CVSSERVICE
+   if (w && w->url().isLocalFile() && !w->isUntitled() && CVSService::ref()->exists())
+   {
+     if (m_cvsMenuId == -1)
+     {
+       m_tabPopup->insertSeparator();
+       m_cvsMenuId = m_tabPopup->insertItem(SmallIcon("cervisia"), i18n("CVS"), CVSService::ref()->menu());
+     }
+     if (Project::ref()->contains(w->url()))
+       CVSService::ref()->setRepository(Project::ref()->projectBaseURL().path());
+     else
+         CVSService::ref()->setRepository(w->url().directory());
+     CVSService::ref()->setCurrentFile(w->url().path());
+   } else
+   if (m_cvsMenuId != -1)
+   {
+       int idx = m_tabPopup->indexOf(m_cvsMenuId);
+       m_tabPopup->removeItemAt(idx-1);
+       m_tabPopup->removeItem(m_cvsMenuId);
+       m_cvsMenuId = -1;
+   }
+#endif
    m_tabPopup->exec(point);
 }
 
@@ -531,4 +733,22 @@ void ViewManager::slotDocumentClosed(const KURL& url)
   emit documentClosed(url);
 }
 
+/** Return the URL of the currently active document */
+QString ViewManager::currentURL()
+{
+  Document *w = activeDocument();
+  if (w)
+  {
+    return w->url().url();
+  } else
+  {
+    QuantaView * view = lastActiveEditorView();
+    if (view)
+      return view->document()->url().url();
+    else
+      return "";
+  }
+}
+
+
 #include "viewmanager.moc"
